<?
class OPENOFFICE {

  private $origFile;
  private $ooFile;
  private $ooname;
  private $fileName;
  private $tmpdir;
  private $prefix;
  private $suffix;
  private $content;
  private $errorMessage;
  private $objectId;
  private $zip;
    
  function __construct($path,$objId=null) {
  
    $this->origFile = $path;
    $this->fileName = array_pop(explode("/",$path));
    $this->objectId = $objId;
    $this->zip = new ZipArchive;
      
    //make our temp directories
    $this->makeTemp();    
    $info = fileInfo($this->origFile);
    $ooext = $info["openoffice"];

    //convert to an openoffice file for processing    
    if ($ooext) $this->ooFile = $this->convert($ooext,$this->origFile);

    //die if we had a problem
    if (!$this->ooFile || !file_exists($this->ooFile)) 
    {
      $this->throwError("Could not create openoffice file from source");
      return false;
    }
    else $this->ooName = array_pop(explode("/",$this->ooFile));

  }

  function throwError($err) {
  
    $this->errorMessage = $err;
    
  }
  
  function error() {
  
    return $this->errorMessage;
    
  }

  function getExtension($filename) {

    $convname = array_pop(explode("/",$filename));
  
    //get our file's current extension
    $pos = strrpos($convname,".");  
    $ext = substr($convname,$pos+1);

    return strtolower($ext);  

  }


  /***************************************************
    make a temp directory for working with the files
  ***************************************************/
  function makeTemp() {

    //make a temp directory for workign with files
    $this->tmpdir = TMP_DIR."/".USER_LOGIN."/openoffice";
    recurmkdir($this->tmpdir);

    //clear it out
    $cmd = "rm ".$this->tmpdir."/*";
    `$cmd`;
    
  }

  /*************************************************************
    get the contents of the file
  *************************************************************/
  function getFileContents() {
  
    //unzip it
    $this->zip->open($this->ooFile);
    $data = $this->zip->getFromName("content.xml");
    $this->zip->close();          
  
    return $data;
  
  }
  
  /*************************************************************
    set the contents of the file
  *************************************************************/
  function setFileContents($data = null) {

    //if not passed data, used the file data stored in the object
    if (!$data) $data = $this->prefix.$this->content.$this->suffix;

    //update the archive with the new contents
    $this->zip->open($this->ooFile);
    $data = $this->zip->addFromString("content.xml",$data);
    $this->zip->close();          

  }

  /*************************************************************
    convert a file from one file type to another
  *************************************************************/
  function convert($newext,$convfile=null) {

    $newext = strtolower($newext);

    if (!$convfile) $convfile = $this->ooFile;
    
    //get our file's current name and extension
    $convname = array_pop(explode("/",$convfile));
    $oldext = $ext = fileExtension($convname);
    if ($ext == "docmgr") {
	$info = fileInfo($convfile);
	$ext = $info["extension"];
    }


    //if our old extension equals our new extension, stop here
    if ($newext==$ext) $dest = $convfile;
    else 
    {

      //if the source is html and the destination is something else, rewrite
      //the images so we can still view them
      if ($ext=="html" && $newext!="html") $this->fixHtmlImages($convfile);

      //add the new extension and put it in our oo temp directory
      $dest = str_replace(".".$oldext,".".$newext,$convname);
      $dest = $this->tmpdir."/".$dest;


      //convert the file to the new extension  
      $cmd = "php bin/fileconvert.php \"".$convfile."\" \"".$dest."\"";
      $return_value = 0;
      exec($cmd, $output = array(), $return_value);
      if($return_value) {
	$stdout = file_get_contents(TMP_DIR."/doc_conv.out");
	$stderr=  file_get_contents(TMP_DIR."/doc_conv.err");
	$this->throwError($stdout . " : " . $stderr);
	return false;
      }


      //if the file is html, we have to do some special stuff
      if ($newext=="html") $this->fixWebDoc($dest);

    }
    
    //if destination exists, return it's path, otherwise return false
    if (file_exists($dest)) return $dest;
    else return false;
  
  }

  /***********************************************************
    split our file content into header,content,and footer
  ***********************************************************/
  function parseFile() {
  
    $data = $this->getFileContents();

    //extract the content
    $begin = "</text:sequence-decls>";
    $end = "</office:text>";

    //get the position for the start and end of our content
    $pos1 = stripos($data,$begin) + strlen($begin);					//content starts after this
    $pos2 = stripos($data,$end);														//content ends after this
    
    //split into sections
    $this->prefix = substr($data,0,$pos1);
    $this->suffix = substr($data,$pos2);
    $this->content = substr($data,$pos1,$pos2-$pos1);
  
  }

  /**********************************************************	
    return contents of the file
  **********************************************************/
  function getContent() {
    return $this->content;
  }

  /**********************************************************
    duplicates a page of the file within the file on a 
    new page
  **********************************************************/
  function duplicate($data) {
  
    //this makes our page break at the page tag
    $break = "<style:style style:name=\"MANUALPAGE\" style:family=\"paragraph\" style:parent-style-name=\"Standard\">";
    $break .= "<style:paragraph-properties fo:break-before=\"page\"/>";
    $break .= "</style:style>";
    $break .= "</office:automatic-styles>";  

    $search = "</office:automatic-styles>";  
    //$pb = "<text:p text:style-name=\"MANUALPAGE\"></text:p>";		removing this shouldn't work, but it does
    $pb = null;
    
    //if our pagebreak style isn't in there, add it
    if (!stristr($this->prefix,$break)) $this->prefix = str_ireplace($search,$break,$this->prefix);

    $this->content = $this->content.$pb.$data;

  }

  /*******************************************************
    set the content of the file
  *******************************************************/  
  function setContent($data) {
    $this->content = $data;
  }
  
  //do some extra stuff to make an html file look right
  function fixWebDoc($file) {

    $data = file_get_contents($file);

    //a fix for extra spacing oo puts in it's converted html pages.  
    //$fix = "<style type=\"text/css\">P {margin:0px;padding:0px}</style>
    //        </head>";

    $data = $this->fixWebImg($data);

    file_put_contents($file,$data);
  
  }
  
  function fixWebImg($data) 
  {

    global $DB;

    if ($this->objectId) 
    {

      //we are converting, so clear out any existing documents and/or create a storage folder
      $opt = null;
      $opt["object_id"] = $this->objectId;
      $opt["clearall"] = "1";

      $d = new DOCMGR_DOCUMENT($opt);
      $d->createStorage();

    } else 
    {

      $d = new DOCMGR_DOCUMENT();
      $d->createTemp();
      
    }

    $storageId = $ret["storage_id"];

    $sessionId = session_id();
    
    $arr = scandir($this->tmpdir);
    $extens = array("png","jpg","gif");

    if (count($arr) > 0) 
    {
    
      foreach ($arr AS $file) 
      {
      
        $ext = fileExtension($file);

        if (in_array($ext,$extens)) 
        {

          $opt = null;
          $opt["parent_id"] = $storageId;
          $opt["name"] = $file;
          $opt["filepath"] = $this->tmpdir."/".$file;

          $d = new DOCMGR_OBJECT($opt);
          $objId = $d->save();
                    
          $path = SITE_URL."app/viewimage.php?objectId=".$objId."&[DOCMGR_SESSION_MARKER]";

          $search = "src=\"".$file."\"";
          $replace = "src=\"".$path."\"";
          $data = str_ireplace($search,$replace,$data);

        }
              
      }
      
    }

    return $data;
    
  }

  //use username/password instead of session ids so openoffice
  //can access the images to do the conversion
  function fixHtmlImages($file) 
  {

    $loginstr = "login=".USER_LOGIN."&password=".USER_PASSWORD;

    $data = file_get_contents($file);
    $data = str_replace("[DOCMGR_SESSION_MARKER]",$loginstr,$data);

    file_put_contents($file,$data);

  }

  /********************************************
    return the thumbnail for our document
  ********************************************/
  function getThumbnail() {

    //unzip it
    $this->zip->open($this->ooFile);
    $data = $this->zip->getFromName("Thumbnails/thumbnail.png");
    $this->zip->close();          

    return $data;

  }

}
