<?php

class APP extends INSTALL_STEP
{
  function __construct() {
    $this->filename = "app-config.php";
    array_push($this->entries,
	    "openoffice_path",
	    "app_ocr",
	    "app_wget",
	    "app_pdftotext",
	    "app_pdfimages",
	    "app_sendmail",
	    "app_php",
	    "app_clamav",
	    "app_tiffinfo",
	    "app_tiffsplit",
	    "app_convert",
	    "app_mogrify",
	    "app_montage",
	    "app_identify"
    );
    array_push($this->emptyEntriesAllow,
      "openoffice_path",
      "app_ocr",
      "app_wget"
    );

  }

  public function display()
  {
    require_once("config/app-config.php");

    $conf = array();
    $conf['openoffice_path'] = importPostC("openoffice_path");
    $conf['app_ocr'] = importPostC("app_ocr");
    $conf['app_wget'] = importPostC("app_wget");
    $conf['app_pdftotext'] = importPostC("app_pdftotext");
    $conf['app_pdfimages'] = importPostC("app_pdfimages");
    $conf['app_sendmail'] = importPostC("app_sendmail");
    $conf['app_php'] = importPostC("app_php");
    $conf['app_clamav'] = importPostC("app_clamav");
    $conf['app_tiffinfo'] = importPostC("app_tiffinfo");
    $conf['app_tiffsplit'] = importPostC("app_tiffsplit");
    $conf['app_convert'] = importPostC("app_convert");
    $conf['app_mogrify'] = importPostC("app_mogrify");
    $conf['app_montage'] = importPostC("app_montage");
    $conf['app_identify'] = importPostC("app_identify");
  
    $content = '<h3>Please enter application paths below.</h3>
	<h2>Site config</h2>

	<p class="config-desc">
	  Apps marked with * are required.  For non-required apps, just leave the field
	  blank to disable them.
	</p>

	<div class="config-section">
	  <div class="config-input">' . aField($conf, 'openoffice_path', 'OpenOffice.org installation path:') . '</div>
	  <p class="config-desc">
	    Leave blank if automatically configured through environment variables or other means.
	  </p>
	</div>
	<p class="config-desc">
	  The apps below may be swapped out for others provided they output to stdout.
	</p>
	<div class="config-section">
	  <div class="config-input">' . aField($conf, 'app_ocr', 'OCR') . '</div>
	  <div class="config-input">' . aField($conf, 'app_wget', 'wget') . '</div>
	</div>
	<p class="config-desc">
	  The apps below cannot be changed.  However, you may specify the full path to the app if it\'s
	  not in a place apache can find it.
	</p>
	<div class="config-section">
	  <div class="config-input">' . aLabel('XPDF', 'http://www.foolabs.com/xpdf/') . '</div>
	  <div class="config-input">' . aField($conf, 'app_pdftotext','pdftotext*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_pdfimages','pdfimages*') . '</div>
	</div>

	<div class="config-section">
	  <div class="config-input">' . aField($conf, 'app_php','php*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_sendmail','sendmail*') . '</div>
	</div>

	<div class="config-section">
	  <div class="config-input">' . aLabel('ClamAV', 'http://www.clamav.net') . '</div>
	  <div class="config-input">' . aField($conf, 'app_clamav','clamscan') . '</div>
	</div>

	<div class="config-section">
	  <div class="config-input">' . aLabel('LibTiff', 'http://www.remotesensing.org/libtiff/') . '</div>
	  <div class="config-input">' . aField($conf, 'app_tiffinfo','tiffinfo*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_tiffsplit','tiffsplit*') . '</div>
	</div>

	<div class="config-section">
	  <div class="config-input">' . aLabel('ImageMagick', 'http://www.imagemagick.org') . '</div>
	  <div class="config-input">' . aField($conf, 'app_convert','convert*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_mogrify','mogrify*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_montage','montage*') . '</div>
	  <div class="config-input">' . aField($conf, 'app_identify','identify*') . '</div>
	</div>
                ';  

    return $content;
    
  }

  protected function testConfig()
  {
    global $errors;

    foreach ($this->entries AS $entry)
    {
      if($errors[$entry])
	continue;

      $app = $_POST[$entry];
      
      //remove any command line arguments
      $arr = explode(" ",$app);
      $app = $arr[0];

      if ($entry=="openoffice_path")
      {

	if ($app)
	{
	  $python = $app."/program/python";

	  //now see if we can find it
	  $output = `which $python`;
	  $output = trim($output);

	  if (!$output)
	  {
	    $errors[$entry] = "Could not find the python application in the OpenOffice.org installation prefix";
	  }

	}
	else
	{
	  $ret = 0;
	  // We'll check if we're able to load the required python module.
	  // This we do within a 'login shell' to get all the standard environment
	  // variables set (ie. most distributions add the path of the module to $PYTHONPATH),
	  // as the web server is usually executed in a shell without all these set...
	  passthru("sh --login -c 'python -c \"import uno\" &>/dev/null'", $ret);
	  if($ret)
	    $errors[$entry] = "Unable to load the required 'uno' python module";
	}
      
      }
      else
      {
	if(!$app)
	  continue;

        //now see if we can find it
        $output = `which $app`;
        $output = trim($output);

	if (!$output)
	{
	  $errors[$entry] = "Apache could not find application \"".$app."\" in ".$_SERVER["PATH"];
	  if(in_array($entry,$this->emptyEntriesAllow))
	    $errors[$entry] .= ", if you don't have/want it, clear the field";
	}
      }
        
    }

  
  }
      
}


