<?php

//required files
include("lib/pgsql.php");
include("lib/misc.php");
require_once("classes/step.php");

function aLabel($name, $url, $urlText = null) {
  $string = "<label class=\"column\">$name</label><a href=\"$url\">" . ($urlText ? $urlText : $url) .  "</a>";
}

function aField(&$conf, $field, $text, $type = "text", $value = "", $onclick = '') {
  static $radioCount = 0;
  $string = "";
  if( $type != "" ) {
    $xtype = "type=\"$type\"";
  } else {
    $xtype = "";
  }

  $id = $field;
  $nolabel = ($type == "radio") || ($type == "hidden");

  if ($type == 'radio')
    $id .= $radioCount++;

  if( !$nolabel ) {
    $string .= "<label class='column' for=\"$id\">$text</label>";
  }

  if(($type == "radio" && $value == $conf[$field]) || ($type == "checkbox" && $conf[$field] == true)) {
    $checked = "checked='checked'";
  } else {
    $checked = "";
  }

  $string .= "<input $xtype name=\"$field\" id=\"$id\" class=\"iput-$type\" $checked ";
  if ($onclick) {
    $string .= " onclick='$onclick' " ;
  }
  $string .= "value=\"";
  if( $type == "radio" ) {
    $string .= htmlspecialchars( $value );
  } else {
    $string .= htmlspecialchars( $conf[$field] );
  }

  $string .= "\" />";
  if( $nolabel ) {
    $string .= "<label for=\"$id\">$text</label>";
  }

  global $errors;
  if(array_key_exists($field, $errors)) {
    //FIXME: ihk, this newline hack is fugly..
    $newline = false;
    if(strlen($errors[$field]) > 25)
      $newline = true;
    if($newline)
      $string .= "<div class='config-error'>\n";
    $string .= "<span class='error'>" . htmlspecialchars( $errors[$field] ) . "</span>\n";
    if($newline)
      $string .= "</div>";
  }
  return $string;
}

//which step are we on
session_start();

//we made it to here, record there's a setup in process
$_SESSION["installInProgress"] = 1;

//our main theme
define("THEME_PATH","themes/default");

//setup our possible config classes
$steps = array();
$stepArr[] = "verify";
$stepArr[] = "config";
$stepArr[] = "database";
$stepArr[] = "files";
$stepArr[] = "app";

//get their files
foreach ($stepArr AS $file)
{
  require_once("classes/".$file.".php");
}

if(!array_key_exists("config", $_SESSION)) {
  $_SESSION["config" ] = array();
}
//start at the beginning
if (!array_key_exists("step", $_SESSION) || $_SESSION["step"] < 0)
{
  $_SESSION["step"] = 0;
  $_SESSION["nextstep"] = 0;
}
else if($_SESSION["step"] >= count($stepArr))
{
  $_SESSION["step"] = count($stepArr)-1;
}

//init our current class based on the step we're on
$curClass = $stepArr[$_SESSION["step"]];
$c = new $curClass();
$finished = false;
$errorMessage = "";
$errors = array();

// Keep all our settings with us for the session..
foreach ($_POST AS $key => $value) {
  if($key == "step" || $key == "nextstep" || $key == "action")
    continue;
  $_SESSION[$curClass][$key] = $value;
}

//if there's a submitted process, handle it
if (array_key_exists("action", $_POST) && $_POST["action"]=="next") 
{

  $c->process();

  //if no errors loads the next step
  $err = $c->getError();
  
  if (!$err && count($errors) == 0) 
  {
    $_SESSION["step"]++;

    //if there's another class to load, load it.
    if ($stepArr[$_SESSION["step"]])
    {

      $curClass = $stepArr[$_SESSION["step"]];
      $c = new $curClass();

    }
    //otherwise bail
    else
    {
      $finished = true;
    
    }

  } else $errorMessage = $err;

}
//go back a page
else if (array_key_exists("action", $_POST) && $_POST["action"]=="back")
{

  if ($_SESSION["step"] > 0)
    $_SESSION["step"]--;

  $curClass = $stepArr[$_SESSION["step"]];
  $c = new $curClass();

}

//setup our main form
$siteContent = "
<form name=\"pageForm\" method=\"post\">
<input type=\"hidden\" name=\"step\" id=\"step\" value=\"".$_SESSION["step"]."\">
<input type=\"hidden\" name=\"action\" id=\"action\" value=\"\">
";

//all done, show the final message
if ($finished) 
{

  $siteContent .= "<div style=\"padding:10px;width:600px;\">
                    <h3>Your setup is complete.</h3>
		    <p> To complete the installation, please do the following:
		    <ol>
	    	      <li>Download config/local/tmp/*.php with your FTP client or file manager</li>
		      <li>Upload the files to the parent directory</li>
		      <li>Delete config/local/tmp/*.php</li>
		      <li>Start using <a href='". SITE_URL ."'>your DocMGR</a>!
		      </ol>
		    <p>If you are in a shared hosting environment, do <strong>not</strong> just move config/local/tmp/*.php
		      remotely. config/local/tmp/*.php is currently owned by the user your webserver is running under,
		      which means that anyone on the same server can modify your configuration! Downloading
		      it and uploading it again will hopefully change the ownership to a user ID specific to you.</p>
		    </p>
		    <p>
		      If you want to run setup again, remove your local configuration files (config/local/*.php) and/or
		      temporary configuration files (config/local/tmp/*.php.
                      You may also safely remove the entire install/ directory if you are done with setup.
                    </p>
                    <p>
                      If this is a new installation, the default username and password is admin/admin.
                    </p>
                    <p>
                      <a href=\"index.php\">Click to login</a>
                    </p>
                  </div>
                  ";

}
//still going, load main config form
else
{

  //toolbar
  $siteContent .= "<div class=\"toolbar\">";

  //give us a back button if not on first page
  if ($_SESSION["step"]!=0) 
  {
    $siteContent .= "<div class=\"toolbarCell\" onclick=\"document.pageForm.action.value='back';document.pageForm.submit();\">
                      <img src=\"".THEME_PATH."/images/icons/back.png\" align=\"left\"> Back
                     </div>
                     ";
  }

  //next button, end toolbar, show class content
  $siteContent .= "		<div class=\"toolbarCell\" onclick=\"document.pageForm.action.value='next';document.pageForm.submit();\">
                        <img src=\"".THEME_PATH."/images/icons/next.png\" align=\"left\"> Next
                      </div>
		      </div>
		      ";
  if ($errorMessage)
    $siteContent .= '<div class="errorMessage">' . htmlspecialchars($errorMessage) . '</div>';
  else if (count($errors))
    $siteContent .= '<div class="errorMessage">Errors occured, make sure everything below is filled out correctly</div>';

  $siteContent .= "<div style=\"width:600px;padding-left:10px;\">
                      ".$c->display()."
                    </div>
                    ";
  

}

//end form
$siteContent .= "</form>";

//load our display template
include("normal.php");

//always stop here to prevent the anything else from loading
die;
