
/**********************************************************************************
	CLASS:	TREE
	PURPOSE:	Builds an explorer-type navigational tree for navigating through 
						collections
**********************************************************************************/

var curtree;

function TREE() {

	this.curval = "";									//array or comma delimited list of selected collections
	this.container = "";							//html element we put the tree in
	this.writecont = "";							//the current container we are writing to
	this.reload = "";									//force a reload of the tree
	this.noexpand = "";								//don't expand root node by default
	this.mode = "link";
	this.formname = "parentId[]";
	this.treeval = new Array();
	this.clicker = "";

	/***************************************************************************
		FUNCTION:	load
		PURPOSE:	loads the tree using supplied parameters
		INPUTS:		opt -> associative keyed array.  Keys match descriptions above
	***************************************************************************/
	this.load = function(opt) {

		//set our container.  init writing to the root	
		this.container = opt.container;
		this.writecont = this.container;
		if (opt.mode) this.mode = opt.mode;
		if (opt.formname) this.formname = opt.formname;
		if (opt.curval) this.treeval = opt.curval.split(",");
		if (opt.click) this.clicker = opt.click;

		clearElement(this.container);

		if (this.mode=="checkbox" || this.mode=="radio") this.createCeilingForm();

		this.browseObject();

	};

	/***************************************************************************
		FUNCTION:	setCurTree
		PURPOSE:	stores a reference to the tree we are working in.  Useful
							if we have multiple trees and they are operated on by something
							outside this class
	***************************************************************************/

	this.setCurTree = function() {

		//copy the reference to the tree we are currently working with (for outside functions to use)
		curtree = this;

	};


	this.createCeilingForm = function() {
	
	  var row = ce("div");
	  row.style.marginLeft = "-5px";
	  if (this.mode=="radio") var cb = createRadio(this.formname,"0");
		else var cb = createCheckbox(this.formname,"0");
	  row.appendChild(cb);
	  row.appendChild(ctnode("Home"));

		//if an onclick is set, set a call to the function passing the event as the parameter
		if (this.clicker) setClick(cb,this.clicker);
	
	  var key = arraySearch("0",this.treeval);
	  if (key!=-1) cb.checked = true;
	
	  this.container.appendChild(row);
	
	};

	/***************************************************************************
		FUNCTION:	writeBrowse
		PURPOSE:	writes the results of our collection browsing
	***************************************************************************/
	this.writeBrowse = function(data) {

		clearSiteStatus();

		 
		//clearElement(this.writecont);

		if (data.error) alert(data.error);
		else {


			//nothing found, don't show anything
			if (data.collection) { 

				//loop through our returns and keep an entry for them
				for (var i=0;i<data.collection.length;i++) {

					//get our entry depending on if we're in form mode or link mode
					if (this.mode=="checkbox" || this.mode=="radio") var row = this.createFormRow(data.collection[i]);
					else var row = this.createLinkRow(data.collection[i]);
					this.writecont.appendChild(row);

				}

			}

		}

	};

	/***************************************************************************
		FUNCTION:	browseObject
		PURPOSE:	shows all collections under the passed one.  if init is passed
							it loads the entire tree to display currently set collections
	***************************************************************************/
	this.browseObject = function(id) {

    		//assemble our command
    		var p = new PROTO();
    		p.add("command","docmgr_search_browsecol");        //browse collections only
    		if (id) p.add("curval",id);          							//currently selected value
				p.post(DOCMGR_API,createMethodReference(this,"writeBrowse"));

	};

	/****************************************************
		FUNCTION: createLinkRow
		PURPOSE:	creates an entry for a collection with
							a link when the name is clicked on
	*****************************************************/
	this.createLinkRow = function(obj) {
	
		var row = ce("div");
		row.setAttribute("projectid",obj.id);
	
		//setup some variables depending on whether or not this is an expanded row
		if (obj.children && obj.children.length > 0 && obj.children[0].collection) {
			var imgsrc = theme_path + "/images/dashbox.gif";
			var foldersrc = theme_path + "/images/open_folder.png";
			var subdisplay = "block";
		} else {
			var imgsrc = theme_path + "/images/plusbox.gif";
			var foldersrc = theme_path + "/images/closed_folder.png";
			var subdisplay = "none";
		}

		//if children, create a link for browsing
		if (isData(obj.child_count) && obj.child_count > 0) {
	
			var img = ce("img");
			img.setAttribute("src",imgsrc);
			img.style.marginRight = "3px";

			row.appendChild(img);
			img.onclick = createMethodReference(this,"cycleObject");
	
		} else {
			row.style.paddingLeft = "13px";
		}
	
		var folderimg = ce("img");
		folderimg.setAttribute("src",foldersrc);
		folderimg.style.marginRight = "3px";
		folderimg.onclick = createMethodReference(this,"cycleObject");
		row.appendChild(folderimg);

		//browse path must exist as a function for this to all work
		var link = ce("a","","",obj.name);
		link.setAttribute("href","javascript:void(0)");

		//if we have a ceiling path, reset the ceiling to that path every time.  this helps when we have multiple trees with different ceilings
		setClick(link,"browseObject(\"" + obj.id + "\")");

		row.appendChild(link);
	
		//create a subdiv for putting children
		var subdiv = ce("div");
		subdiv.style.display = subdisplay;
		subdiv.style.paddingLeft = "13px";
	
		//if data below here, expand and add as well
		if (obj.children && obj.children.length > 0 && obj.children[0].collection) {
			subdiv.style.display = "block";
			for (var i=0;i<obj.children[0].collection.length;i++) {
				subdiv.appendChild(this.createLinkRow(obj.children[0].collection[i]));
			}
	
		}
	
		row.appendChild(subdiv);
	
		return row;
	
	};

	/****************************************************
	  FUNCTION: createFormRow
	  PURPOSE:  creates an entry for a collection with
	            a link when the name is clicked on
	*****************************************************/
	this.createFormRow = function(obj) {
	
	  var row = ce("div");
	  row.setAttribute("projectid",obj.id);

	  //setup some variables depending on whether or not this is an expanded row
	  if (obj.children && obj.children[0].collection) {
	    var imgsrc = theme_path + "/images/dashbox.gif";
	    var subdisplay = "block";
	  } else {
	    var imgsrc = theme_path + "/images/plusbox.gif";
	    var subdisplay = "none";
	  }
	   
	  //if children, create a link for browsing
	  if (isData(obj.child_count) && obj.child_count > 0) {
	
	    var img = ce("img");
	    img.setAttribute("src",imgsrc);
	    img.style.marginRight = "3px"; 
	    row.appendChild(img);

			img.onclick = createMethodReference(this,"cycleObject");
	
	  } else {
	    row.style.paddingLeft = "12px";
	  }
	   
	  //form element
	  if (this.mode=="checkbox") var cb = createCheckbox(this.formname,obj.id);
	  else if (this.mode=="radio") var cb = createRadio(this.formname,obj.id); 
	
	  cb.style.marginRight = "3px";
	
	  row.appendChild(cb);

		//if an onclick is set, set a call to the function passing the event as the parameter
		if (this.clicker) setClick(cb,this.clicker);
	
	  //look to see if this is a selected project and it needs to be checked
	  if (this.treeval.length > 0) {
	    var key = arraySearch(obj.id,this.treeval);
	    if (key!=-1) cb.checked = true;
	  }
	   
	  row.appendChild(ctnode(obj.name));
	
	  //create a subdiv for putting children
	  var subdiv = ce("div");
	  subdiv.style.display = subdisplay;
	  subdiv.style.paddingLeft = "13px";
	
	  //if data below here, expand and add as well
	  if (obj.children && obj.children[0].collection) {
	    subdiv.style.display = "block";
	    for (var i=0;i<obj.children[0].collection.length;i++) {
	      subdiv.appendChild(createFormRow(obj.children[0].collection[i]));
	    }
	
	  }
	   
	  row.appendChild(subdiv);
	
	  return row;
	
	};

	/***************************************************************************
		FUNCTION:	cycleObject
		PURPOSE:	shows or hides collections under a parent collection
	***************************************************************************/

	this.cycleObject = function(e) {

		var ref = getEventSrc(e).parentNode;
		var objid = ref.getAttribute("projectid");

		//now reference the first sub div under it
		this.writecont = ref.getElementsByTagName("div")[0];

		var imgarr = ref.getElementsByTagName("img");
		var img1 = imgarr[0];
		var img2 = imgarr[1];

		if (!this.writecont.hasChildNodes()) {

			this.writecont.style.display = "block";

			//if it's a folder, just make it open
			if (img1.getAttribute("src").indexOf("box.gif")!=-1) 
				img1.setAttribute("src",theme_path + "/images/dashbox.gif");
			else
				img1.setAttribute("src",theme_path + "/images/open_folder.png");
	
			if (img2 && img2.getAttribute("src").indexOf("box.gif")==-1) {
				img2.setAttribute("src",theme_path + "/images/open_folder.png");
			}

			//load the child collections of this folder
			this.browseObject(objid);
	
		} else {
	
			clearElement(this.writecont);
			img1.setAttribute("src",theme_path + "/images/plusbox.gif");
			this.writecont.style.display = "none";
	
			if (img2 && img2.getAttribute("src").indexOf("box.gif")==-1) {
				img2.setAttribute("src",theme_path + "/images/closed_folder.png");
			}
	
		}
	
	};


	/***************************************************************************
		FUNCTION:	cycleObjectPath
		PURPOSE:	shows or hides collections under a parent collection.  this one
							can be referenced by outside functions to browse the tree
	***************************************************************************/
	this.cycleObjectPath = function(objpath,forceopen) {
	
		var arr = this.container.getElementsByTagName("div");
		var run = 0;
	
		for (var i=0;i<arr.length;i++) {
	
			//we found our element that made the call
			if (arr[i].getAttribute("projectpath")==objpath) {

				run = 1;			//we did something
	
				//now reference the first sub div under it
				this.writecont = arr[i].getElementsByTagName("div")[0];
	
				var imgarr = arr[i].getElementsByTagName("img");
	
				if (!this.writecont.hasChildNodes() || forceopen) {
	
					this.writecont.style.display = "block";
					var src = imgarr[0].getAttribute("src");
	
					//if it's a folder, just make it open
					if (src.indexOf("box.gif")!=-1) {				
						imgarr[0].setAttribute("src",theme_path + "/images/dashbox.gif");
					} else {
						imgarr[0].setAttribute("src",theme_path + "/images/open_folder.png");
					}
	
					if (imgarr[1] && imgarr[1].getAttribute("src").indexOf("box.gif")==-1) {
						imgarr[1].setAttribute("src",theme_path + "/images/open_folder.png");
					}
	
					//if it hasn't been loaded before, load it
					this.browseObject(objpath);
	
				} else {
	
					clearElement(this.writecont);
					imgarr[0].setAttribute("src",theme_path + "/images/plusbox.gif");
					this.writecont.style.display = "none";

					if (imgarr[1] && imgarr[1].getAttribute("src").indexOf("box.gif")==-1) {
						imgarr[1].setAttribute("src",theme_path + "/images/closed_folder.png");
					}
	
				}
	
				break;
	
			}
	
		}

		//if nothing was done, reload the tree
		if (run==0) {

			var opt = this;
			opt.curval = objpath;	
			opt.reload = 1;
			opt.noexpand = "";	
			this.load(opt);

		}
	
	};
		
}
	
	